unit IEInstVer;

{
  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  IEInstVer
  Copyright  2001 Ch. Bergmann
  All rights reserved.

  ------------------------------------------------------------------------------

  All contents of this file and all other files included in the IEInstVer
  archive are Copyright  2001 Ch. Bergmann - AS NOT OTHERWISE MARKED.

  Use and/or distribution of them requires acceptance of the following
  License Agreement.

  ------------------------------------------------------------------------------

  LICENSE AGREEMENT 1.2:

  "Author" herein refers Ch. Bergmann (the creator of the IEInstVer component).
  "Software" refers to all files bearing this notice, as well as any other
  files and source code included with IEInstVer (typically part of a .zip
  archive), and all content in them, regardless of whether any modifications
  have been made.

  1) This notice may not be removed or altered from any source distribution.

  2) The origin of this software must not be misrepresented, you must not
     claim that you wrote the original software.

  3) The Software may NOT be redistributed in ANY FORM (including but not
     limited to .DPK, .DCR, .PAS, .DCU) after you have obtained it (typically
     by downloading from the Authors site) without the express written
     permission of the Author.

  4) You are permitted to compile the Software into any kind of applications
     (Freeware, Shareware, Commercial).
     If you use this software in a product, an acknowledgment in the product
     documentation or the product itself would be appreciated but is not
     required.

  5) This software is provided "AS-IS", without any express or implied
     warranty. In no event shall the Author be held liable for any damages
     arising from the use of/inability to use this software.

  6) Full backward compatibility in future versions of the Software is not
     guaranteed. In no event shall the Author be held liable for any
     inconvenience or damages arising from lack of backward compatibility.

  7) The Author may terminate your rights to use the Software for any reason
     at his sole discretion.

  8) If you do not agree to any of these terms, you are not permitted to use
     the Software in any way, and all copies of it must be deleted from your
     system(s).

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
}

interface

uses
  Windows, Messages, Classes, Forms, Dialogs, Registry, SysUtils;

type
  TIEInstVer = class(TComponent)
  private
    FAbout: string;
    FTriggerCr: Boolean;
    FTriggerUp: Boolean;
    FUpd: Boolean;
    FIE1: Boolean;
    FIE2: Boolean;
    FIE3: Boolean;
    FIE4: Boolean;
    FIE5: Boolean;
    FIE55: Boolean;
    FIE6: Boolean;
    FInstalled: Boolean;
    FShVersionNumber: string;
    FVersionNumber: string;
    FBuild: string;
    FVersionDescription: string;
    FEncryption: string;
    FWindowTitle: string;
    FPreviousVersionNumber: string;
    FPreviousVersionDescription: string;
    FCookies: string;
    FFavorites: string;
    FHistory: string;
    FCache: string;
    FDownload: string;
    FLinks: string;
    FSave: string;
    FStartPage: string;
    FSearchPage: string;
    FLocalPage: string;
    FUserAgent: string;

    FOnIE1: TNotifyEvent;
    FOnIE2: TNotifyEvent;
    FOnIE3: TNotifyEvent;
    FOnIE4: TNotifyEvent;
    FOnIE5: TNotifyEvent;
    FOnIE55: TNotifyEvent;
    FOnIE6: TNotifyEvent;
    FOnNotIE1: TNotifyEvent;
    FOnNotIE2: TNotifyEvent;
    FOnNotIE3: TNotifyEvent;
    FOnNotIE4: TNotifyEvent;
    FOnNotIE5: TNotifyEvent;
    FOnNotIE55: TNotifyEvent;
    FOnNotIE6: TNotifyEvent;
    FOnInstalled: TNotifyEvent;
    FOnNotInstalled: TNotifyEvent;
    FOnUpd: TNotifyEvent;

    function GetSystemDir: string;
    function GetDllVersion(Dll, IType: string): string;
    function GetIEVersion: string;
    function GetPreviousVersion: string;
    function GetDescription(VerNr: string; UpDate: Boolean): string;
    function GetEncryption: string;
    function GetUserAgent: string;
    function GetShellPath(FolderName: string): string;
    function GetDLDir: string;
    function GetLinksDir: string;
    function GetMisc(Key: string): string;
    procedure GetNoneS(Value: string);
    procedure GetData;

  protected
    procedure Loaded; override;

  public
    constructor Create(AOwner: TComponent); override;
    procedure TriggerEvents;
    procedure UpdateValues;

    property IEInstalled: Boolean read FInstalled;
    property IE1Installed: Boolean read FIE1;
    property IE2Installed: Boolean read FIE2;
    property IE3Installed: Boolean read FIE3;
    property IE4Installed: Boolean read FIE4;
    property IE5Installed: Boolean read FIE5;
    property IE5_5Installed: Boolean read FIE55;
    property IE6Installed: Boolean read FIE6;
    property VersionNumberRegistry: string read FVersionNumber;
    property BuildNumberRegistry: string read FBuild;
    property VersionNumberShDocVwDll: string read FShVersionNumber;
    property VersionDescription: string read FVersionDescription;
    property EncryptionStrength: string read FEncryption;
    property WindowTitle: string read FWindowTitle;
    property UserAgent: string read FUserAgent;
    property PreviousVersionNumber: string read FPreviousVersionNumber;
    property PreviousVersionDescription: string read
    FPreviousVersionDescription;
    property CookiesFolderPath: string read FCookies;
    property FavoritesFolderPath: string read FFavorites;
    property HistoryFolderPath: string read FHistory;
    property InternetCacheFolderPath: string read FCache;
    property DownloadFolderPath: string read FDownload;
    property LinksFolderPath: string read FLinks;
    property SaveFolderPath: string read FSave;
    property LocalPage: string read FLocalPage;
    property SearchPage: string read FSearchPage;
    property StartPage: string read FStartPage;

  published
    property About: string read FAbout write GetNoneS stored False;
    property TriggerEventsOnLoaded: Boolean read FTriggerCr write FTriggerCr
    default true;
    property TriggerEventsOnUpdate: Boolean read FTriggerUp write FTriggerUp
    default true;

    property OnIEInstalled: TNotifyEvent read FOnInstalled write FOnInstalled;
    property OnIE1Installed: TNotifyEvent read FOnIE1 write FOnIE1;
    property OnIE2Installed: TNotifyEvent read FOnIE2 write FOnIE2;
    property OnIE3Installed: TNotifyEvent read FOnIE3 write FOnIE3;
    property OnIE4Installed: TNotifyEvent read FOnIE4 write FOnIE4;
    property OnIE5Installed: TNotifyEvent read FOnIE5 write FOnIE5;
    property OnIE5_5Installed: TNotifyEvent read FOnIE55 write FOnIE55;
    property OnIE6Installed: TNotifyEvent read FOnIE6 write FOnIE6;
    property OnNotIEInstalled: TNotifyEvent read FOnNotInstalled write
    FOnNotInstalled;
    property OnNotIE1Installed: TNotifyEvent read FOnNotIE1 write FOnNotIE1;
    property OnNotIE2Installed: TNotifyEvent read FOnNotIE2 write FOnNotIE2;
    property OnNotIE3Installed: TNotifyEvent read FOnNotIE3 write FOnNotIE3;
    property OnNotIE4Installed: TNotifyEvent read FOnNotIE4 write FOnNotIE4;
    property OnNotIE5Installed: TNotifyEvent read FOnNotIE5 write FOnNotIE5;
    property OnNotIE5_5Installed: TNotifyEvent read FOnNotIE55 write FOnNotIE55;
    property OnNotIE6Installed: TNotifyEvent read FOnNotIE6 write FOnNotIE6;
    property OnUpdateValues: TNotifyEvent read FOnUpd write FOnUpd;

  end;

procedure Register;

implementation

uses DsgnIntf;

const
  IEIVersion = '4.0';
  IEIDate = '2001/06/11';

  {
    -!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!-

     Microsoft, Windows, Office are registered trademarks of Microsoft Corp.
     Internet Explorer is trademark of Microsoft Corp.

     Microsoft Internet Explorer Version Information are
     Copyright  2001 Microsoft Corp. and taken from:

     Q164539 - How to Determine Which Version of Internet Explorer is Installed
     http://support.microsoft.com/support/kb/articles/q164/5/39.asp
     and
     Q165871 - How to Determine Your Previous Version of Internet Explorer
     http://support.microsoft.com/support/kb/articles/Q165/8/71.asp

    -!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!--!-
  }

  MS = 'Microsoft ';
  MI = 'Internet Explorer ';
  MW = 'Windows ';
  MO = 'Office ';

  IE1_01 = MS + MI + '1.0 (' + MS + MW + '95 Plus! (eta))';

  IE2_01 = MS + MI + '2.0 (' + MS + MW + '95)';
  IE2_02 = MS + MI + '2.0 (' + MS + MW + 'NT)';

  IE3_00 = MS + MI + '3';
  IE3_01 = MS + MI + '3.0';
  IE3_02 = MS + MI + '3.0 (OSR 2)';
  IE3_03 = MS + MI + '3.01';
  IE3_04 = MS + MI + '3.02(a)';

  IE4_00 = MS + MI + '4';
  IE4_01 = MS + MI + '4.0 (Public Preview 2)';
  IE4_02 = MS + MI + '4.0';
  IE4_03 = MS + MI + '4.01';
  IE4_04 = MS + MI + '4.01 (SP 1)';
  IE4_05 = MS + MI + '4.01 (SP 2)';
  IE4_06 = MS + MI +
    '4.x ("Server-Side Page Reference Redirect" Issue Update Installed.)';

  IE5_00 = MS + MI + '5';
  IE5_01 = MS + MI + '5 Developer Preview (eta 1)';
  IE5_02 = MS + MI + '5 (eta/eta 2)';
  IE5_03 = MS + MI + '5';
  IE5_03a = MS + MI + '5 (' + MS + MW + '98)'; //-chb undocumented
  IE5_04 = MS + MI + '5 (' + MS + MO + '2000)';
  IE5_05 = MS + MI + '5.01 (' + MS + MW + '2000 (eta 3), Build 5.00.2031)';
  IE5_06 = MS + MI + '5 (' + MS + MW + '98 SE)';
  IE5_07 = MS + MI +
    '5 ("Malformed Favorites Icon" Security Issue Update Installed.)';
  IE5_08 = MS + MI +
    '5 ("ImportExport Favorites()" Security Issue Update Installed.)';
  IE5_09 = MS + MI +
    '5.0 ("Server-Side Page Reference Redirect" Issue Update Installed.)';
  IE5_10 = MS + MI + '5.01 (' + MS + MW + '2000 (RC 1), Build 5.00.2072)';
  IE5_11 = MS + MI + '5.01 (' + MS + MW + '2000 (RC 2), Build 5.00.2128)';
  IE5_12 = MS + MI + '5.01 (' + MS + MO + '2000 (SR-1))';
  IE5_13 = MS + MI +
    '5.01 ("Server-side Page Reference Redirect" Issue Update Installed.)';
  IE5_14 = MS + MI + '5.01 (' + MS + MW +
    '2000 (RC 3), Build 5.00.2183 (2195))'; //-chb (undocumented)
  IE5_15 = MS + MI + '5.01 (SP 1) (' + MS + MW + '2000)';
  IE5_16 = MS + MI + '5.01 (SP 1) (' + MS + MW + '95/98 or NT 4.0)';

  IE55_00 = MS + MI + '5.5';
  IE55_01 = MS + MI + '5.5 (Developer Preview) (eta) (' + MS + MW +
    'ME (eta 2), Build 4.90.2419)'; //-chb (undocumented)
  IE55_02 = MS + MI + '5.5 & Internet Tools (eta)';
  IE55_03 = MS + MI + '5.5 (' + MS + MW + 'ME, Build 4.90.3000)';
  IE55_04 = MS + MI + '5.5';
  IE55_05 = MS + MI + '5.5 Advanced Security Privacy (eta)';
  IE55_06 = MS + MI + '5.5 (SP 1)';

  IE6_00 = MS + MI + '6';
  IE6_01 = MS + MI + '6 (Public Preview) (eta)';

  IE6x_00 = MS + MI + '6.x';

  CRG = #67#111#112#121#114#105#103#104#116#32#169#32#50#48#48#49#32#32#67 +
    #104#46#32#66#101#114#103#109#97#110#110#46;
  ARR = #65#108#108#32#82#105#103#104#116#115#32#82#101#115#101#114#118#101 +
    #100#46;
  MSG = #73#69#73#110#115#116#86#101#114#32 + IEIVersion + #32#32#40 +
    IEIDate + #41#10#13#10#13 + CRG + #10#13 + ARR +
    #10#13#10#13#121#122#111#64#103#109#120#46#110#101#116#10#13#104#116 +
    #116#112#58#47#47#121#97#122#110#111#46#116#114#105#112#111#100#46#99 +
    #111#109#47#105#101#105#110#115#116#118#101#114#47;

type
  TDoubleInst = class(Exception);

  TAboutComponentEditor = class(TDefaultEditor)
  public
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
    procedure ExecuteVerb(Index: Integer); override;
    procedure Edit; override;
  end;

  TAboutPropertyEditor = class(TPropertyEditor)
  public
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

function TIEInstVer.GetSystemDir: string;
var
  Len: DWORD;
  SysDir: PChar;
  S: string;
begin
  S := 'n/a';
  try
    Len := MAX_PATH + 1;
    GetMem(SysDir, Len);
    try
      if Windows.GetSystemDirectory(SysDir, Len) <> 0 then S := SysDir;
    finally
      FreeMem(SysDir)
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetDllVersion(Dll, IType: string): string;
var
  Info, InfoData, LangPtr: Pointer;
  InfoSize: LongInt;
  Len: DWORD;
  FName: Pchar;
  Infotype, S: string;
begin
  S := 'n/a';
  try
    Len := MAX_PATH + 1;
    FName := Pchar(GetSystemDir + '\' + Dll);
    InfoType := IType;
    InfoSize := GetFileVersionInfoSize(Fname, Len);
    if (InfoSize > 0) then
      begin
        GetMem(Info, InfoSize);
        try
          if GetFileVersionInfo(FName, Len, InfoSize, Info) then
            begin
              Len := 255;
              if VerQueryValue(Info, '\VarFileInfo\Translation', LangPtr, Len)
                then
                InfoType := Format('\StringFileInfo\%0.4x%0.4x\%s'#0,
                  [LoWord(LongInt(LangPtr^)), HiWord(LongInt(LangPtr^)),
                  InfoType]);
              if VerQueryValue(Info, Pchar(InfoType), InfoData, len) then
                S := strPas(InfoData)
            end;
        finally
          FreeMem(Info, InfoSize)
        end
      end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetIEVersion: string;
var
  Reg: TRegistry;
  Ver, S: string;
begin
  S := 'n/a';
  Ver := FShVersionNumber;
  try
    Reg := TRegistry.Create;
    with Reg do
      try
        RootKey := HKEY_LOCAL_MACHINE;
        OpenKey('Software\Microsoft\Internet Explorer', False);
        //IE4/5/6
        if ValueExists('Version') and not (Ver = 'n/a') and not
          (copy(Ver, 0, 4) = '4.70') then
          S := ReadString('Version')
        else
          //IE1/2/3
          begin
            if ValueExists('IVer') then
              begin
                //IE1
                if ReadString('IVer') = '100.' then
                  S := '4.40.210/308';
                //IE2 WinNT
                if (ReadString('IVer') = '101.') then
                  S := '4.40.520-NT';
                //IE2 Win95
                if (ValueExists('Build')) and (ReadString('IVer')
                  = '102.') and (ReadString('Build') = '520') then
                  S := '4.40.520-95';
                //IE3
                if (ValueExists('Build')) and (ReadString('IVer')
                  = '103.') then
                  S := '4.70.' + ReadString('Build')
              end
          end;
        //Build
        if (ValueExists('Build')) and not (length(ReadString('Build')) = 0)
          then
          FBuild := ReadString('Build')
        else
          FBuild := 'n/a'
      finally
        Reg.CloseKey;
        Reg.Free
      end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetPreviousVersion: string;
var
  Reg: TRegistry;
  Key, S: string;
begin
  S := 'n/a';
  try
    if (FIE4) or (FIE5) or (FIE6) then
      begin
        if FIE4 then key := 'Software\Microsoft\IE4\Setup';
        if (FIE5) or (FIE6) then key := 'Software\Microsoft\IE Setup\Setup';
        Reg := TRegistry.Create;
        with reg do
          try
            RootKey := HKEY_LOCAL_MACHINE;
            OpenKey(Key, false);
            if (ValueExists('PreviousIESysFile')) and not
              (ReadString('PreviousIESysFile') = '0.0.0.0') then
              S := ReadString('PreviousIESysFile')
          finally
            CloseKey;
            Free
          end
      end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetDescription(VerNr: string; UpDate: Boolean): string;
var
  S: string;
begin
  S := 'n/a';
  try
    if not (VerNr = 'n/a') then
      begin
        if UpDate then FInstalled := true;

        //IE1
        if VerNr = '4.40.210/308' then
          begin
            if UpDate then FIE1 := true;
            S := IE1_01;
          end;

        //IE2
        if copy(VerNr, 0, 8) = '4.40.520' then
          begin
            if UpDate then FIE2 := true;
            if VerNr = '4.40.520-95' then S := IE2_01;
            if VerNr = '4.40.520-NT' then S := IE2_02;
            if UpDate then FVersionNumber := '4.40.520';
          end;

        //IE3
        if copy(VerNr, 0, 4) = '4.70' then
          begin
            if UpDate then FIE3 := true;
            S := IE3_00;
            if VerNr = '4.70.1155' then S := IE3_01;
            if VerNr = '4.70.1158' then S := IE3_02;
            if VerNr = '4.70.1215' then S := IE3_03;
            if VerNr = '4.70.1300' then S := IE3_04;
          end;

        //IE4
        if (copy(VerNr, 0, 4) = '4.71') or (copy(VerNr, 0, 4) = '4.72')
          then
          begin
            if UpDate then FIE4 := true;
            S := IE4_00;
            if VerNr = '4.71.1008.3' then S := IE4_01;
            if VerNr = '4.71.1712.5' then S := IE4_02;
            if VerNr = '4.72.2106.7' then S := IE4_03;
            if VerNr = '4.72.3110.3' then S := IE4_04;
            if VerNr = '4.72.3612.1707' then S := IE4_05;
            if VerNr = '4.72.3711.2900' then S := IE4_06;
          end;

        //IE5
        if VerNr[1] = '5' then
          begin
            if UpDate then FIE5 := true;
            S := IE5_00;
            if VerNr[3] = '0' then
              begin
                if copy(VerNr, 0, 4) = '5.00' then
                  VerNr := copy(VerNr, 6, length(VerNr) - 5)
                else
                  if copy(VerNr, 0, 4) = '5.0.' then
                    VerNr := copy(VerNr, 5, length(VerNr) - 4);
                if VerNr = '0518.5' then S := IE5_01;
                if VerNr = '0910.1308' then S := IE5_02;
                if VerNr = '2014.0213' then S := IE5_03;
                if VerNr = '2014.213' then S := IE5_03;
                if VerNr = '2014.0216' then S := IE5_03a; //-chb undocumented
                if VerNr = '2014.216' then S := IE5_03a; //-chb undocumented
                if VerNr = '2314.1000' then S := IE5_04;
                if VerNr = '2516.1900' then S := IE5_05;
                if VerNr = '2614.3500' then S := IE5_06;
                if VerNr = '2717.2000' then S := IE5_07;
                if VerNr = '2721.1400' then S := IE5_08;
                if VerNr = '2723.2900' then S := IE5_09;
                if VerNr = '2919.0800' then S := IE5_10;
                if VerNr = '2919.800' then S := IE5_10;
                if VerNr = '2919.3800' then S := IE5_11;
                if VerNr = '2919.6307' then S := IE5_12;
                if VerNr = '2919.6400' then S := IE5_13;
                if VerNr = '2920.0000' then S := IE5_14;
                if VerNr = '2920.0' then S := IE5_14;
                if VerNr = '3103.1000' then S := IE5_15;
                if VerNr = '3105.0106' then S := IE5_16;
                if VerNr = '3105.106' then S := IE5_16;
              end
            else
              begin
                //IE5.5
                if UpDate then FIE55 := true;
                S := IE55_00;
                if VerNr = '5.50.3825.1300' then S := IE55_01;
                if VerNr = '5.50.4030.2400' then S := IE55_02;
                //if VerNr = '5.50.4133.2400' then S := IE55_xx; //-chb undocumented
                if VerNr = '5.50.4134.0100' then S := IE55_03;
                if VerNr = '5.50.4134.100' then S := IE55_03;
                if VerNr = '5.50.4134.0600' then S := IE55_04;
                if VerNr = '5.50.4134.600' then S := IE55_04;
                if VerNr = '5.50.4308.2900' then S := IE55_05;
                if VerNr = '5.50.4522.1800' then S := IE55_06;
              end
          end;

        //IE6
        if VerNr[1] = '6' then
          begin
            if UpDate then FIE6 := true;
            S := IE6_00;
            if VerNr[3] = '0' then
              begin
                if copy(VerNr, 0, 4) = '6.00' then
                  VerNr := copy(VerNr, 6, length(VerNr) - 5)
                else
                  if copy(VerNr, 0, 4) = '6.0.' then
                    VerNr := copy(VerNr, 5, length(VerNr) - 4);
                if VerNr = '2462.0000' then S := IE6_01;
                if VerNr = '2462.0' then S := IE6_01;
              end
            else
              //IE6.x
              begin
                //if UpDate then FIE6x := true;
                S := IE6x_00;
                // if VerNr = '6.xx.xxxx.xxxx' then S := IE6x_xx;
              end;
          end
      end;
    Result := S
  except
    Result := 'n/a';
    FInstalled := false;
    FIE1 := false;
    FIE2 := false;
    FIE3 := false;
    FIE4 := false;
    FIE5 := false;
    FIE55 := false;
    FIE6 := false
  end
end;

function TIEInstVer.GetEncryption: string;
var
  DllVer, S: string;
begin
  S := 'n/a';
  if FIE3 or FIE4 then
    begin
      DllVer := GetDllVersion('schannel.dll', 'FileDescription');
      if DllVer = 'PCT/SSL Security Provider (Export Version)' then
        S := '40-bit or Server Gated Cryptography (SGC)';
      if DllVer = 'PCT/SSL Security Provider (US and Canada Use Only)' then
        S := '128-bit';
      exit
    end;
  //-chb undocumented and not yet fully tested
  if FIE5 then
    begin
      DllVer := GetDllVersion('schannel.dll', 'FileDescription');
      //IE5
      //Contributed by Andrew Plus <aeplus2001@yahoo.com.au>
      if DllVer = 'TLS / SSL Security Provider (Export Version)' then
        S := '40-bit or Server Gated Cryptography (SGC)';
      if DllVer = 'TLS / SSL Security Provider (US/Canada Only, Not for Export)'
        then S := '128-bit';
      //IE5.5
      if DllVer = 'TLS / SSL Security Provider' then S := '128-bit';
    end;
  Result := S
end;

function TIEInstVer.GetUserAgent: string;
var
  reg: TRegistry;
  S: string;
begin
  S := 'n/a';
  try
    Reg := TRegistry.Create;
    with reg do
      try
        RootKey := HKEY_CURRENT_USER;
        OpenKey('\Software\Microsoft\Windows\CurrentVersion\Internet Settings\',
          false);
        if (ValueExists('User Agent')) and not (length(ReadString('User Agent'))
          = 0) then
          S := ReadString('User Agent')
      finally
        CloseKey;
        Free
      end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetShellPath(FolderName: string): string;
var
  Reg: TRegistry;
  S: string;
begin
  S := 'n/a';
  try
    Reg := TRegistry.Create;
    with reg do
      try
        Rootkey := HKEY_CURRENT_USER;
        OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', false);
        if (ValueExists(FolderName)) and not (length(ReadString(FolderName)) = 0)
          then
          S := ReadString(FolderName)
      finally
        CloseKey;
        Free
      end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetDLDir: string;
var
  reg: TRegistry;
  S: string;
begin
  S := 'n/a';
  try
    Reg := TRegistry.Create;
    with Reg do
      try
        RootKey := HKEY_CURRENT_USER;
        OpenKey('Software\Microsoft\Internet Explorer', False);
        if (ValueExists('Download Directory')) and not
          (length(ReadString('Download Directory')) = 0) then
          S := ReadString('Download Directory')
      finally
        CloseKey;
        Free
      end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetLinksDir: string;
var
  reg: TRegistry;
  S: string;
begin
  S := 'n/a';
  try
    if FFavorites <> 'n/a' then
      begin
        Reg := TRegistry.Create;
        with Reg do
          try
            RootKey := HKEY_CURRENT_USER;
            OpenKey('Software\Microsoft\Internet Explorer\Toolbar', False);
            if (ValueExists('LinksFolderName')) and not
              (length(ReadString('LinksFolderName')) = 0) then
              S := FFavorites + '\' + ReadString('LinksFolderName')
          finally
            CloseKey;
            Free
          end
      end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetMisc(Key: string): string;
var
  reg: TRegistry;
  S: string;
begin
  S := 'n/a';
  try
    Reg := TRegistry.Create;
    with Reg do
      try
        RootKey := HKEY_CURRENT_USER;
        OpenKey('Software\Microsoft\Internet Explorer\Main', False);
        if (ValueExists(Key)) and not (length(ReadString(Key)) = 0) then
          S := ReadString(Key)
      finally
        CloseKey;
        Free
      end;
    Result := S
  except
    Result := 'n/a'
  end
end;

procedure TIEInstVer.GetNoneS(Value: string);
begin {}
end;

procedure TIEInstVer.GetData;
begin
  FIE1 := false;
  FIE2 := false;
  FIE3 := false;
  FIE4 := false;
  FIE5 := false;
  FIE55 := false;
  FIE6 := false;
  FInstalled := false;
  FShVersionNumber := GetDllVersion('shdocvw.dll', 'FileVersion');
  FVersionNumber := GetIEVersion;
  if FVersionNumber <> 'n/a' then
    FVersionDescription := GetDescription(FVersionNumber, true)
  else
    FVersionDescription := GetDescription(FShVersionNumber, true);
  FEncryption := GetEncryption;
  FWindowTitle := GetMisc('Window Title');
  FPreviousVersionNumber := GetPreviousVersion;
  FPreviousVersionDescription := GetDescription(FPreviousVersionNumber, false);
  FCookies := GetShellPath('Cookies');
  FFavorites := GetShellPath('Favorites');
  FHistory := GetShellPath('History');
  FCache := GetShellPath('Cache');
  FDownload := GetDLDir;
  FLinks := GetLinksDir;
  FSave := GetMisc('Save Directory');
  FLocalPage := GetMisc('Local Page');
  FSearchPage := GetMisc('Search Page');
  FStartPage := GetMisc('Start Page');
  FUserAgent := GetUserAgent
end;

procedure TIEInstVer.TriggerEvents;
begin
  if not (csDesigning in ComponentState) then
    begin
      if (FIE1) and (Assigned(FOnIE1)) then FOnIE1(Self);
      if (FIE2) and (Assigned(FOnIE2)) then FOnIE2(Self);
      if (FIE3) and (Assigned(FOnIE3)) then FOnIE3(Self);
      if (FIE4) and (Assigned(FOnIE4)) then FOnIE4(Self);
      if (FIE5) and (Assigned(FOnIE5)) then FOnIE5(Self);
      if (FIE55) and (Assigned(FOnIE55)) then FOnIE55(Self);
      if (FIE6) and (Assigned(FOnIE6)) then FOnIE6(Self);
      if not (FIE1) and (Assigned(FOnNotIE1)) then FOnNotIE1(Self);
      if not (FIE2) and (Assigned(FOnNotIE2)) then FOnNotIE2(Self);
      if not (FIE3) and (Assigned(FOnNotIE3)) then FOnNotIE3(Self);
      if not (FIE4) and (Assigned(FOnNotIE4)) then FOnNotIE4(Self);
      if not (FIE5) and (Assigned(FOnNotIE5)) then FOnNotIE5(Self);
      if not (FIE55) and (Assigned(FOnNotIE55)) then FOnNotIE55(Self);
      if not (FIE6) and (Assigned(FOnNotIE6)) then FOnNotIE6(Self);
      if (FInstalled) and (Assigned(FOnInstalled)) then FOnInstalled(Self);
      if not (FInstalled) and (Assigned(FOnNotInstalled)) then
        FOnNotInstalled(Self);
      if (FUpd) and (Assigned(FOnUpd)) then
        try
          FOnUpd(Self)
        finally
          FUpd := false
        end
    end
end;

procedure TIEInstVer.UpdateValues;
begin
  GetData;
  if FTriggerUp then
    begin
      FUpd := true;
      TriggerEvents
    end
end;

constructor TIEInstVer.Create(AOwner: TComponent);
var
  i: word;
  CompCount: byte;
begin
  inherited Create(AOwner);

  CompCount := 0;
  if (csDesigning in ComponentState) then
    begin
      with (AOwner) do
        begin
          for i := 0 to ComponentCount - 1 do
            if Components[i] is TIEInstVer then inc(CompCount);

          if CompCount > 1 then
            raise
              TDoubleInst.Create('There already exists an Instance of IEInstVer on '
              + AOwner.Name)
        end
    end;

  FAbout := #73#69#73#110#115#116#86#101#114#32 + IEIVersion + #32#32#40 +
    IEIDate + #41#32#45#32 + CRG + #32 + ARR;
  FTriggerCr := True;
  FTriggerUp := True;
  GetData
end;

procedure TIEInstVer.Loaded;
begin
  inherited Loaded;
  if FTriggerCr then TriggerEvents;
end;

function TAboutComponentEditor.GetVerb(Index: Integer): AnsiString;
begin
  Result := '&About'
end;

function TAboutComponentEditor.GetVerbCount: Integer;
begin
  Result := 1
end;

procedure TAboutComponentEditor.ExecuteVerb(Index: Integer);
begin
  if not (Index = 0) then exit;
  edit
end;

procedure TAboutComponentEditor.Edit;
begin
  MessageDlg(MSG, mtInformation, [mbOK], 0)
end;

function TAboutPropertyEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly]
end;

function TAboutPropertyEditor.GetValue: string;
begin
  Result := '[DoubleClick]'
end;

procedure TAboutPropertyEditor.Edit;
begin
  MessageDlg(MSG, mtInformation, [mbOK], 0)
end;

procedure Register;
begin
  RegisterComponents('yComps', [TIEInstVer]);
  RegisterComponentEditor(TIEInstVer, TAboutComponentEditor);
  RegisterPropertyEditor(TypeInfo(string), TIEInstVer, 'About',
    TAboutPropertyEditor);
end;

end.
